package services

import (
	"PaymentCenter/app/constants/common"
	"PaymentCenter/app/constants/errorcode"
	"PaymentCenter/app/http/entities"
	"PaymentCenter/app/http/entities/backend"
	"PaymentCenter/app/http/entities/front"
	"PaymentCenter/app/models/orderrequestlogmodel"
	"PaymentCenter/app/models/ordersmodel"
	"PaymentCenter/app/models/paychannelmodel"
	"PaymentCenter/app/utils"
	"PaymentCenter/config"
	"encoding/json"
	"fmt"
	"net/url"
	"strconv"
	"time"
	"xorm.io/builder"
)

// 收银台支付渠道列表
func PayPageChannelList(reqParam front.PayChannelListRequest) (resultPayChannelList []paychannelmodel.PayChannel, code int) {

	orderId, err := strconv.ParseInt(reqParam.OrderId, 10, 64)
	if err != nil {
		code = errorcode.ParamError
		return
	}

	// 订单检查
	orderInfo := &ordersmodel.Orders{Id: orderId}
	orderInfo, code = OrderFindOne(orderInfo, builder.Eq{"id": orderId})
	if code != errorcode.Success {
		return
	}
	if !(orderInfo.Status == common.ORDER_STATUS_PAYING || orderInfo.Status == common.ORDER_STATUS_WAITPAY) {
		code = errorcode.OrderStatusErr
		return
	}

	// 商户拥有的支付渠道检查
	req := backend.PayChannelListRequest{
		MerchantId:  orderInfo.MerchantId,
		PageRequest: entities.PageRequest{},
	}
	payList := make([]paychannelmodel.PayChannel, 0)
	payList, _, code = PayChannelList(req)
	if code != errorcode.Success {
		return
	}
	if len(payList) == 0 {
		code = errorcode.PayChannelNotFound
		return
	}
	merchantPayChannelMap := make(map[int]paychannelmodel.PayChannel, 0)
	for _, pay := range payList {
		merchantPayChannelMap[pay.ChannelType] = pay
	}

	// 客户端环境检查
	openType := ClientEnvCheck(reqParam.UserAgent)
	if channels, ok := common.OpenInPayChannelMap[openType]; !ok {
		code = errorcode.ClientEnvErr
		return
	} else {
		for _, channel := range channels {
			if _, ok = merchantPayChannelMap[channel]; ok {
				resultPayChannelList = append(resultPayChannelList, merchantPayChannelMap[channel])
			}
		}
	}

	return resultPayChannelList, code
}

// 客户端环境检查
func ClientEnvCheck(ua string) int {
	if utils.IsWeChatClient(ua) {
		return common.OpenInWeChat
	} else if utils.IsPC(ua) {
		return common.OpenInWindows
	} else if utils.IsMobile(ua) {
		return common.OpenInMobile
	}

	return common.OpenInUnknown
}

// 预支付
type payUrl struct {
	ClientIp string
	param    front.PayReqsV2
	app      *AppCheck
	result   front.PayReqsV2Response
}

func NewPayUrl(param front.PayReqsV2) *payUrl {
	return &payUrl{
		param: param,
	}
}
func (this *payUrl) WithApp(app *AppCheck) *payUrl {
	this.app = app
	return this
}
func (this *payUrl) WithClientIp(ip string) *payUrl {
	this.ClientIp = ip
	return this
}

// 订单是否是支付状态
func orderStatusCheck(order ordersmodel.Orders) (code int) {
	switch order.Status {
	case common.ORDER_STATUS_CLOSE:
		code = errorcode.OrderClosed
	case common.ORDER_STATUS_PAYED:
		code = errorcode.OrderPayed
	case common.ORDER_STATUS_FAILED:
		code = errorcode.OrderFailed
	case common.ORDER_STATUS_WAITPAY, common.ORDER_STATUS_PAYING:
		code = errorcode.Success
	default:
		code = errorcode.OrderStatusErr
	}
	return code
}

func (this *payUrl) saveOrder() {
	order := &ordersmodel.Orders{
		MerchantId: this.app.App.MerchantId,
		//PayChannelId: w.PayParam.Channel.Id,
		AppId:      this.app.App.Id,
		OutTradeNo: this.param.OutTradeNo,
		OrderType:  common.ORDER_TYPE_PAY,
		Amount:     this.param.Amount,
		ExtJson:    this.param.ExtJson,
		Desc:       this.param.Desc,
		Status:     common.ORDER_STATUS_WAITPAY,
	}
	this.result.Order, this.result.PayCode = OrderCreate(order)
}

func (this *payUrl) PayUrlV2Service() (result front.PayReqsV2Response, code int) {
	var (
		channelList []paychannelmodel.PayChannel
		order       = new(ordersmodel.Orders)
	)

	// redis 分布式锁 订单号幂等
	key := utils.GetRealKey("payUrl:" + this.param.OutTradeNo)
	lockValue := fmt.Sprintf("%d", time.Now().UnixNano())
	ok, err := utils.AcquireLock(key, lockValue, time.Second*time.Duration(3))
	if ok {
		defer utils.ReleaseLock(key, lockValue)
	} else {
		if err != nil {
			utils.Log(nil, "", "PayUrlV2Service,获取分布式锁失败", fmt.Sprintf("错误原因：%s", err.Error()))
		}
		return
	}

	// 商户是否有配置支付渠道
	channelList, _, code = PayChannelList(backend.PayChannelListRequest{MerchantId: this.app.App.MerchantId})
	if code != errorcode.Success {
		return
	}
	if len(channelList) == 0 {
		code = errorcode.PayChannelNotFound
		return
	}
	// 订单是否存在
	conn := builder.NewCond()
	conn = conn.And(builder.Eq{"out_trade_no": this.param.OutTradeNo})
	conn = conn.And(builder.Eq{"app_id": this.param.AppId})
	order, code = OrderFindOne(order, conn)
	if code == errorcode.Success {
		// 订单存在
		this.result.Order = order
		this.result.PayCode = orderStatusCheck(*order)
	} else if code == errorcode.OrdersNotFound {
		// 订单不存在
		this.saveOrder()
	} else {
		return
	}
	code = this.result.PayCode
	if code == errorcode.Success {
		this.result.Url = config.GetConf().PayService.Host + common.PayPageAddress + "?id=" + strconv.FormatInt(this.result.Order.Id, 10)
	}

	return this.result, code
}

// 拼接微信授权链接  appid:微信的公众号appid
func GetWxAuthUrl(appid string, orderId int64) (targetUrl string, code int) {
	var (
		// 重定向地址
		redirectUri = config.GetConf().PayService.Host + common.WXCodeRedirectAddress
	)
	baseUrl := "https://open.weixin.qq.com/connect/oauth2/authorize"
	redirectUri = url.QueryEscape(redirectUri)
	responseType := "code"
	scope := "snsapi_base"
	order := strconv.Itoa(int(orderId))

	targetUrl = baseUrl + "?appid=" + appid + "&redirect_uri=" + redirectUri + "&response_type=" + responseType + "&scope=" + scope + "&state=" + order + "#wechat_redirect"
	code = errorcode.Success
	return
}

// 收银台获取付款链接
func GetPayLinkService(req front.GetPayLinkRequest) (result string, code int) {
	var (
		payChannel      = &paychannelmodel.PayChannel{}
		order           = &ordersmodel.Orders{}
		orderPayRequest front.PayReqsV2
	)
	// 支付方式校验
	payChannel, code = PayChannelFindOne(payChannel, builder.Eq{"id": req.PayChannelId})
	if code != errorcode.Success {
		return
	}
	if payChannel.ExpireTime.Unix() < time.Now().Unix() {
		code = errorcode.PayChannelExpired
		return
	}
	// 订单校验
	order, code = OrderFindOne(order, builder.Eq{"id": req.OrderId})
	if code != errorcode.Success {
		return
	}
	// 订单状态校验
	code = orderStatusCheck(*order)
	if code != errorcode.Success {
		return
	}
	// 获取订单的请求参数
	orderRequestLog := orderrequestlogmodel.OrderRequestLog{
		OutTradeNo: order.OutTradeNo,
		AppId:      order.AppId,
	}
	has, err := OrderRequestLogs(&orderRequestLog)
	if err != nil {
		code = handErr(err)
		return
	}
	if !has {
		code = errorcode.OrderPayRequestLogNotExist
		return
	}
	err = json.Unmarshal([]byte(orderRequestLog.MerchantRequest), &orderPayRequest)
	if err != nil {
		code = handErr(err)
		return
	}
	// 支付方式需要用户授权
	if payChannel.ChannelType == common.PAY_CHANNEL_WECHAT_JSAPI && orderPayRequest.OpenId == "" {
		result, code = GetWxAuthUrl(payChannel.AppId, order.Id)
		return
	}
	// 请求第三方支付获取付款链接
	//payParam := paymentService.PayOrderRequest{
	//	OrderId: order.Id,
	//}
	//res := paymentService.PaymentService(context.Background(), payParam)
	//result = res.Result
	//code = res.Code
	return
}
